@@ -50,6 +50,7 @@ typedef struct {
 
 	guint reg_tries;
 	guint init_tries;
+	gboolean init_retried;
 	gboolean init_ok;
 	guint pin_tries;
 
@@ -219,18 +220,18 @@ real_do_dial (NMGsmDevice *device, guint cid)
 }
 
 static void
-set_apn_done (NMSerialDevice *device,
-              int reply_index,
-              const char *reply,
-              gpointer user_data)
+real_set_apn_done (NMGsmDevice *self,
+                   int reply_index,
+                   const char *reply,
+                   guint cid)
 {
 	switch (reply_index) {
 	case 0:
-		NM_GSM_DEVICE_GET_CLASS (device)->do_dial (NM_GSM_DEVICE (device), GPOINTER_TO_UINT (user_data));
+		NM_GSM_DEVICE_GET_CLASS (self)->do_dial (self, cid);
 		break;
 	default:
 		nm_warning ("Setting APN failed");
-		nm_device_state_changed (NM_DEVICE (device),
+		nm_device_state_changed (NM_DEVICE (self),
 		                         NM_DEVICE_STATE_FAILED,
 		                         NM_DEVICE_STATE_REASON_GSM_APN_FAILED);
 		break;
@@ -238,13 +239,22 @@ set_apn_done (NMSerialDevice *device,
 }
 
 static void
-set_apn (NMGsmDevice *device)
+set_apn_done (NMSerialDevice *device,
+              int reply_index,
+              const char *reply,
+              gpointer user_data)
+{
+	NM_GSM_DEVICE_GET_CLASS (device)->set_apn_done (NM_GSM_DEVICE (device), reply_index, reply, GPOINTER_TO_UINT (user_data));
+}
+
+void
+nm_gsm_device_set_apn (NMGsmDevice *device)
 {
 	NMGsmDevicePrivate *priv = NM_GSM_DEVICE_GET_PRIVATE (device);
 	NMSettingGsm *setting;
 	char *command;
 	const char *apn;
-	const char *responses[] = { "OK", "ERROR", NULL };
+	const char *responses[] = { "OK", "ERROR", "+CME ERROR: GPRS - a profile (cid) is currently active", NULL };
 	guint cid = 1;
 
 	priv->reg_tries = 0;
@@ -291,7 +301,7 @@ manual_registration_response (NMSerialDevice *device,
 
 	switch (reply_index) {
 	case 0:
-		set_apn (NM_GSM_DEVICE (device));
+		nm_gsm_device_set_apn (NM_GSM_DEVICE (device));
 		break;
 	case -1:
 		/* Some cards (ex. Sierra AC860) don't immediately respond to commands
@@ -346,7 +356,7 @@ get_network_response (NMSerialDevice *device,
 		break;
 	}
 
-	set_apn (NM_GSM_DEVICE (device));
+	nm_gsm_device_set_apn (NM_GSM_DEVICE (device));
 }
 
 static void
@@ -538,10 +548,17 @@ init_full_done (NMSerialDevice *device,
 		power_up (NM_GSM_DEVICE (device));
 		break;
 	case -1:
-		nm_warning ("Modem second stage initialization timed out");
-		nm_device_state_changed (NM_DEVICE (device),
-		                         NM_DEVICE_STATE_FAILED,
-		                         NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED);
+		/* Retry the init string once */
+		if (priv->init_retried) {
+			nm_warning ("Modem second stage initialization timed out");
+			nm_device_state_changed (NM_DEVICE (device),
+			                         NM_DEVICE_STATE_FAILED,
+			                         NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED);
+		} else {
+			nm_info ("Retrying modem initialization (%d)", priv->init_tries);
+			init_modem_full (self);
+			priv->init_retried = TRUE;
+		}
 		break;
 	default:
 		if (priv->init_ok) {
@@ -664,6 +681,13 @@ enter_pin (NMGsmDevice *device, NMGsmSecret secret_type, gboolean retry)
 		g_free (command);
 	} else {
 		nm_info ("(%s): GSM %s secret required", nm_device_get_iface (NM_DEVICE (device)), secret_name);
+
+		/* Close the serial device as we don't know how long it will take for
+		 * secrets to return, and the port gets re-opened by stage1 when they
+		 * do.
+		 */
+		nm_serial_device_close (NM_SERIAL_DEVICE (device));
+
 		nm_device_state_changed (NM_DEVICE (device),
 		                         NM_DEVICE_STATE_NEED_AUTH,
 		                         NM_DEVICE_STATE_REASON_NONE);
@@ -776,6 +800,7 @@ init_done (NMSerialDevice *device,
 	switch (reply_index) {
 	case 0:
 		priv->init_ok = TRUE;
+		priv->init_retried = FALSE;
 		check_pin (NM_GSM_DEVICE (device));
 		break;
 	case 1:
@@ -783,10 +808,16 @@ init_done (NMSerialDevice *device,
 		schedule_init_modem_again (NM_GSM_DEVICE (device));
 		break;
 	case -1:
-		nm_warning ("Modem initialization timed out");
-		nm_device_state_changed (NM_DEVICE (device),
-		                         NM_DEVICE_STATE_FAILED,
-		                         NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED);
+		if (priv->init_retried) {
+			nm_warning ("Modem initialization timed out");
+			nm_device_state_changed (NM_DEVICE (device),
+			                         NM_DEVICE_STATE_FAILED,
+			                         NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED);
+		} else {
+			nm_info ("Retrying modem initialization (%d)", priv->init_tries);
+			init_modem (NM_SERIAL_DEVICE (device));
+			priv->init_retried = TRUE;
+		}
 		break;
 	default:
 		priv->init_tries++;
@@ -830,6 +861,7 @@ real_act_stage1_prepare (NMDevice *device, NMDeviceStateReason *reason)
 
 	NM_GSM_DEVICE_GET_PRIVATE (device)->init_tries = 0;
 	NM_GSM_DEVICE_GET_PRIVATE (device)->init_ok = FALSE;
+	NM_GSM_DEVICE_GET_PRIVATE (device)->init_retried = FALSE;
 	NM_GSM_DEVICE_GET_PRIVATE (device)->pin_tries = 0;
 	NM_GSM_DEVICE_GET_PRIVATE (device)->reg_tries = 0;
 
@@ -1183,6 +1215,7 @@ nm_gsm_device_class_init (NMGsmDeviceClass *klass)
 	device_class->deactivate_quickly = real_deactivate_quickly;
 
 	klass->do_dial = real_do_dial;
+	klass->set_apn_done = real_set_apn_done;
 
 	serial_class->get_ppp_name = real_get_ppp_name;
 
