@@ -76,7 +76,9 @@ typedef struct {
 
 GType nm_manager_get_type (void);
 
-NMManager *nm_manager_get (void);
+NMManager *nm_manager_get (const char *state_file,
+                           gboolean initial_net_enabled,
+                           gboolean initial_wifi_enabled);
 
 /* Device handling */
 
